// const Node = {
//     addChild(child) {
//         this.children.push(child);
//         return child;
//     }
// }


export function tag(name, attrs, children) {
    return {
        type: 'tag',
        tag: name,
        attrs: attrs || {},
        children: children || [],

        addTag(name, attrs, children) {
            const child = tag(name, attrs, children);
            this.children.push(child);
            return child;
        },
    };
}


export function text(content) {
    return {
        type: 'text',
        text: content,
    };
}


export function serialize(node, depth) {
    if (!depth) {
        depth = 0;
    }

    const indent = ' '.repeat(depth * 4);
    let fragments = [];

    // version tag, if this is the top level
    if (depth === 0) {
        fragments.push('<?xml version="1.0" encoding="UTF-8"?>\n')
    }

    fragments.push(`${indent}<${node.tag}`);

    // this happens if there are multiple text nodes within the same parent
    if (node.type === 'text') {
        return `${indent}${escape(node.text)}`;
    }

    if (node.children === undefined) {
        console.log(node);
    }

    // opening tag <element attr="value">
    for (const attr in node.attrs) {
        fragments.push(` ${attr}="${node.attrs[attr]}"`);
    }
    if (node.children.length === 0) {
        fragments.push(' />');
        return fragments.join('');
    }
    fragments.push('>');

    // if the only child is a single text node, skip recursion and just dump contents directly
    if (node.children.length === 1 && node.children[0].type === 'text') {
        const text = escape(node.children[0].text);
        fragments.push(text);
    }
    // otherwise, start a new line for each child node, then recurse
    else {
        for (const child of node.children) {
            fragments.push('\n');
            fragments.push(serialize(child, depth + 1));
        }
        // no need to verify that there were children, we already did that
        fragments.push(`\n${indent}`);
    }

    fragments.push(`</${node.tag}>`);

    return fragments.join('');
}


function escape(text) {
    // we aren't going to bother with escaping attributes, so we won't worry about quotes
    return text
        .replaceAll('&', '&amp;')
        .replaceAll('<', '&lt;')
        .replaceAll('>', '&gt;');
}
